@@ -142,7 +142,7 @@ int main(int argc, char *argv[]) {
 
 				if (prop <= 0.0 || prop >= 1.0) {
 					soft_errx("A probability should be a value between 0 and "
-							  "1, exlusive; Ignoring -p %f argument.",
+							  "1, exclusive; Ignoring -p %f argument.",
 							  prop);
 					break;
 				}
@@ -199,17 +199,17 @@ int main(int argc, char *argv[]) {
 				break;
 			}
 			case 'm': {
-				// valid options are 'RAW' and 'JC'
 				if (strcasecmp(optarg, "RAW") == 0) {
 					MODEL = M_RAW;
 				} else if (strcasecmp(optarg, "JC") == 0) {
 					MODEL = M_JC;
 				} else if (strcasecmp(optarg, "KIMURA") == 0) {
 					MODEL = M_KIMURA;
+				} else if (strcasecmp(optarg, "LOGDET") == 0) {
+					MODEL = M_LOGDET;
 				} else {
-					soft_errx(
-						"Ignoring argument for --model. Expected Raw, JC or "
-						"Kimura");
+					soft_errx("Ignoring argument for --model. Expected Raw, "
+							  "JC, Kimura or LogDet");
 				}
 				break;
 			}
@@ -340,14 +340,13 @@ void usage(int status) {
 		"\tUse '-' as file name to read from stdin.\n"
 		"Options:\n"
 		"  -b, --bootstrap=INT  Print additional bootstrap matrices\n"
-		"      --file-of-filenames=FILE  Read additional filenames from "
-		"FILE; one per line\n"
+		"      --file-of-filenames=FILE  Read additional filenames from FILE; "
+		"one per line\n"
 		"  -j, --join           Treat all sequences from one file as a single "
 		"genome\n"
 		"  -l, --low-memory     Use less memory at the cost of speed\n"
 		"  -m, --model=MODEL    Pick an evolutionary model of 'Raw', 'JC', "
-		"'Kimura'; default: "
-		"JC\n"
+		"'Kimura', 'LogDet'; default: JC\n"
 		"  -p FLOAT             Significance of an anchor; default: 0.025\n"
 		"      --progress=WHEN  Print a progress bar 'always', 'never', or "
 		"'auto'; default: auto\n"
