@@ -80,7 +80,8 @@ double model_coverage(const model *MM) {
  */
 double estimate_RAW(const model *MM) {
 	size_t nucl = model_total(MM);
-	size_t SNPs = model_sum(MM, AtoC, AtoG, AtoT, CtoG, CtoT, GtoT);
+	size_t SNPs = model_sum(MM, AtoC, AtoG, AtoT, CtoA, CtoG, CtoT, GtoA, GtoC,
+							GtoT, TtoA, TtoC, TtoG);
 
 	// Insignificant results. All abort the fail train.
 	if (nucl <= 3) {
@@ -111,8 +112,9 @@ double estimate_JC(const model *MM) {
  */
 double estimate_KIMURA(const model *MM) {
 	size_t nucl = model_total(MM);
-	size_t transitions = model_sum(MM, AtoG, CtoT);
-	size_t transversions = model_sum(MM, AtoC, AtoT, GtoC, GtoT);
+	size_t transitions = model_sum(MM, AtoG, GtoA, CtoT, TtoC);
+	size_t transversions =
+		model_sum(MM, AtoC, CtoA, AtoT, TtoA, GtoC, CtoG, GtoT, TtoG);
 
 	double P = (double)transitions / (double)nucl;
 	double Q = (double)transversions / (double)nucl;
@@ -124,6 +126,77 @@ double estimate_KIMURA(const model *MM) {
 	return dist <= 0.0 ? 0.0 : dist;
 }
 
+/** @brief computes the evolutionary distance using LogDet.
+ *
+ * The LogDet distance between sequence X and and sequence Y
+ * is given as
+ *
+ * -(1 / K) * (log(det(Fxy)) - 0.5 * log(det(Fxx * Fyy)))
+ *
+ * Where K is the number of character states, Fxy is the site-pattern
+ * frequency matrix, and diagonal matrices Fxx and Fyy give the
+ * frequencies of the different character states in sequences X and Y.
+ *
+ * Each i,j-th entry in Fxy is the proportion of homologous sites
+ * where sequences X and Y have character states i and j, respectively.
+ *
+ * For our purposes, X is the Subject (From) sequence and Y is the
+ * Query (To) sequence and matrix Fxy looks like
+ *
+ *  To   A  C  G  T
+ * From
+ *  A  (            )
+ *  C  (            )
+ *  G  (            )
+ *  T  (            )
+ *
+ * @param MM - The mutation matrix.
+ * @returns The LogDet distance.
+ */
+double estimate_LOGDET(const model *MM) {
+
+	double nucl = (double)model_total(MM);
+	double P[MUTCOUNTS];
+	for (int i = 0; i < MUTCOUNTS; i++) {
+		P[i] = MM->counts[i] / nucl;
+	}
+
+	double logDetFxxFyy =
+		// log determinant of diagonal matrix of row sums
+		log(model_sum(MM, AtoA, AtoC, AtoG, AtoT) / nucl) +
+		log(model_sum(MM, CtoA, CtoC, CtoG, CtoT) / nucl) +
+		log(model_sum(MM, GtoA, GtoC, GtoG, GtoT) / nucl) +
+		log(model_sum(MM, TtoA, TtoC, TtoG, TtoT) / nucl) +
+		// log determinant of diagonal matrix of column sums
+		log(model_sum(MM, AtoA, CtoA, GtoA, TtoA) / nucl) +
+		log(model_sum(MM, AtoC, CtoC, GtoC, TtoC) / nucl) +
+		log(model_sum(MM, AtoG, CtoG, GtoG, TtoG) / nucl) +
+		log(model_sum(MM, AtoT, CtoT, GtoT, TtoT) / nucl);
+
+	// determinant of the site-pattern frequency matrix
+	double detFxy =
+		P[AtoA] * P[CtoC] * (P[GtoG] * P[TtoT] - P[TtoG] * P[GtoT]) -
+		P[AtoA] * P[CtoG] * (P[GtoC] * P[TtoT] - P[TtoC] * P[GtoT]) +
+		P[AtoA] * P[CtoT] * (P[GtoC] * P[TtoG] - P[TtoC] * P[GtoG]) -
+
+		P[AtoC] * P[CtoA] * (P[GtoG] * P[TtoT] - P[TtoG] * P[GtoT]) +
+		P[AtoC] * P[CtoG] * (P[GtoA] * P[TtoT] - P[TtoA] * P[GtoT]) -
+		P[AtoC] * P[CtoT] * (P[GtoA] * P[TtoG] - P[TtoA] * P[GtoG]) +
+
+		P[AtoG] * P[CtoA] * (P[GtoC] * P[TtoT] - P[TtoC] * P[GtoT]) -
+		P[AtoG] * P[CtoC] * (P[GtoA] * P[TtoT] - P[TtoA] * P[GtoT]) +
+		P[AtoG] * P[CtoT] * (P[GtoA] * P[TtoC] - P[TtoA] * P[GtoC]) -
+
+		P[AtoT] * P[CtoA] * (P[GtoC] * P[TtoG] - P[TtoC] * P[GtoG]) +
+		P[AtoT] * P[CtoC] * (P[GtoA] * P[TtoG] - P[TtoA] * P[GtoG]) -
+		P[AtoT] * P[CtoG] * (P[GtoA] * P[TtoC] - P[TtoA] * P[GtoC]);
+
+	double dist = -0.25 * (log(detFxy) - 0.5 * logDetFxxFyy);
+
+	// fix negative zero
+	return dist <= 0.0 ? 0.0 : dist;
+}
+
 /** @brief Bootstrap a mutation matrix.
  *
  * The classical bootstrapping process, as described by Felsenstein, resamples
@@ -132,25 +205,17 @@ double estimate_KIMURA(const model *MM) {
  * resample the elements of the mutation matrix. See Klötzl & Haubold (2016)
  * for details. http://www.mdpi.com/2075-1729/6/1/11/htm
  *
- * @param MM - The original mutation matrix.
+ * @param datum - The original mutation matrix.
  * @returns A bootstrapped mutation matrix.
  */
-model model_bootstrap(const model MM) {
-	model datum = MM;
-
-	size_t nucl = model_total(&MM);
+model model_bootstrap(model datum) {
+	size_t nucl = model_total(&datum);
 	double p[MUTCOUNTS];
 	for (size_t i = 0; i < MUTCOUNTS; ++i) {
-		p[i] = MM.counts[i] / (double)nucl;
+		p[i] = datum.counts[i] / (double)nucl;
 	}
 
-	unsigned int n[MUTCOUNTS];
-
-	gsl_ran_multinomial(RNG, MUTCOUNTS, nucl, p, n);
-
-	for (size_t i = 0; i < MUTCOUNTS; ++i) {
-		datum.counts[i] = n[i];
-	}
+	gsl_ran_multinomial(RNG, MUTCOUNTS, nucl, p, datum.counts);
 
 	return datum;
 }
@@ -216,7 +281,7 @@ void model_count_equal(model *MM, const char *S, size_t len) {
  * @param c - input nucleotide
  * @returns 2bit representation.
  */
-char nucl2bit(char c) {
+char nucl2bit(unsigned char c) {
 	c &= 6;
 	c ^= c >> 1;
 	return c >> 1;
@@ -244,25 +309,13 @@ void model_count(model *MM, const char *S, const char *Q, size_t len) {
 
 		// Pick the correct two bits representing s and q.
 		unsigned char foo = nucl2bit(s);
-		unsigned char baz = nucl2bit(q);
-
-		/*
-		 * The mutation matrix is symmetric. For convenience we define the
-		 * mutation XtoY with X > Y.
-		 */
-		if (baz > foo) {
-			int temp = foo;
-			foo = baz;
-			baz = temp;
-		}
+		unsigned char bar = nucl2bit(q);
 
 		/*
 		 * Finally, we want to map the indices to the correct mutation. This is
 		 * done by utilising the mutation types in model.h.
 		 */
-		static const unsigned int map4 = 0x9740;
-		unsigned int base = (map4 >> (4 * baz)) & 0xf;
-		unsigned int index = base + (foo - baz);
+		unsigned int index = (foo << 2) + bar;
 
 		local_counts[index]++;
 	}
