@@ -10,112 +10,18 @@
 #include "io.h"
 #include "model.h"
 #include "sequence.h"
-#include <assert.h>
 #include <math.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
 
 #ifdef _OPENMP
 #include <omp.h>
 #endif
 
-double shustring_cum_prob(size_t x, double g, size_t l);
 int calculate_bootstrap(const struct model *M, const seq_t *sequences,
 						size_t n);
 
-/**
- * @brief Calculates the minimum anchor length.
- *
- * Given some parameters calculate the minimum length for anchors according
- * to the distribution from Haubold et al. (2009).
- *
- * @param p - The probability with which an anchor will be created under a
- * random model.
- * @param g - The the relative amount of GC in the subject.
- * @param l - The length of the subject (includes revcomp).
- * @returns The minimum length of an anchor.
- */
-size_t min_anchor_length(double p, double g, size_t l) {
-	size_t x = 1;
-
-	while (shustring_cum_prob(x, g / 2, l) < 1 - p) {
-		x++;
-	}
-
-	return x;
-}
-
-/**
- * @brief Calculates the binomial coefficient of n and k.
- *
- * We could (and probably should) use gsl_sf_lnchoose(xx,kk) for this.
- *
- * @param n - The n part of the binomial coefficient.
- * @param k - analogue.
- * @returns (n choose k)
- */
-size_t binomial_coefficient(size_t n, size_t k) {
-	if (n <= 0 || k > n) {
-		return 0;
-	}
-
-	if (k == 0 || k == n) {
-		return 1;
-	}
-
-	if (k > n - k) {
-		k = n - k;
-	}
-
-	size_t res = 1;
-
-	for (size_t i = 1; i <= k; i++) {
-		res *= n - k + i;
-		res /= i;
-	}
-
-	return res;
-}
-
-/**
- * @brief Given `x` this function calculates the probability of a shustring
- * with a length less or equal to `x` under a random model. This means, it is
- * the cumulative probability.
- *
- * Let X be the longest shortest unique substring (shustring) at any position.
- * Then this function computes P{X <= x} with respect to the given parameter
- * set. See Haubold et al. (2009). Note that `x` includes the final mismatch.
- * Thus, `x` is `match length + 1`.
- *
- * @param x - The maximum length of a shustring.
- * @param p - The half of the relative amount of GC in the DNA.
- * @param l - The length of the subject.
- * @returns The probability of a certain shustring length.
- */
-double shustring_cum_prob(size_t x, double p, size_t l) {
-	double xx = (double)x;
-	double ll = (double)l;
-	size_t k;
-
-	double s = 0.0;
-
-	for (k = 0; k <= x; k++) {
-		double kk = (double)k;
-		double t = pow(p, kk) * pow(0.5 - p, xx - kk);
-
-		s += pow(2, xx) * (t * pow(1 - t, ll)) *
-			 (double)binomial_coefficient(x, k);
-		if (s >= 1.0) {
-			s = 1.0;
-			break;
-		}
-	}
-
-	return s;
-}
-
 typedef _Bool bool;
 #define false 0
 #define true !false
@@ -229,18 +135,17 @@ static inline bool anchor(const struct context *ctx,
  * @param query - The actual query string.
  * @param query_length - The length of the query string. Needed for speed
  * reasons.
- * @param gc - The relative GC content of the subject.
+ * @param threshold - Minimal length for an anchor.
  * @returns A matrix with estimates of base substitutions.
  */
 model dist_anchor(const esa_s *C, const char *query, size_t query_length,
-				  double gc) {
+				  size_t threshold) {
 	struct model ret = {.seq_len = query_length, .counts = {0}};
 
 	struct anchor this_match = {0};
 	struct anchor last_match = {0};
 	bool last_was_right_anchor = false;
-
-	size_t threshold = min_anchor_length(ANCHOR_P_VALUE, gc, C->len);
+	size_t border = C->len / 2;
 
 	struct context ctx = {C, query, query_length, threshold};
 
@@ -256,7 +161,8 @@ model dist_anchor(const esa_s *C, const char *query, size_t query_length,
 			size_t end_Q = last_match.pos_Q + last_match.length;
 			// Check if this can be a right anchor to the last one.
 			if (this_match.pos_S > end_S &&
-				this_match.pos_Q - end_Q == this_match.pos_S - end_S) {
+				this_match.pos_Q - end_Q == this_match.pos_S - end_S &&
+				(this_match.pos_S < border) == (last_match.pos_S < border)) {
 
 				// classify nucleotides in the left qanchor
 				model_count_equal(&ret, query + last_match.pos_Q,
